% zgain.m
% PRIOR KNOWLEDGE IN FACTOR STRUCTURE ESTIMATION
%
% Designed : P. Nedoma
% Updated  : November 20024
% Projects : GACR 102/03/0049, AVCR S1075351, 1ET100750401,
%            MINERVA 110330-CP-1-2003-1-ES-MINERVA-M

prodini
echo off
idef = [];

while 1 % ==============================================

if isempty(idef)
ndat  = 300;                          % sample size       
covu  = 1;
covy  = 0.001;  
seed  = 3;      
gain  = [0.9 1.1];
ord   = 6;
end

clc

fprintf('\nPRIOR KNOWLEDGE IN FACTOR STRUCTURE ESTIMATION\n');
fprintf('=======================================\n');
fprintf('\n--- System: SISO dynamic, 2nd order, open loop -------\n');
fprintf('output noise covariance ->   covy   = %g;\n',      covy);
fprintf('output noise covariance ->   covu   = %g;\n',      covu);
fprintf('\n--- Measurements---------------------------------\n');
fprintf('data length             ->   ndat  = %i;\n',       ndat);
fprintf('seed                    ->   seed  = %i;\n',       seed);
fprintf('\n--- Structure estimation ------------------------\n');
fprintf('richest model order     ->   ord   = %g;\n',    ord);
fprintf('\n--- Prior knowledge -----------------------------\n');
fprintf('gain range              ->   gain = [%g %g];\n',   gain);
% fprintf('-------------------------------------------------\n');

dial1;

% test of arguments ===================================
while covy < 0 | isstr(covy)
    fprintf('noise covariance (output) must be a positive number\n')
    covy = input('covy = ')
end

while covu < 0 | isstr(covu)
    fprintf('noise covariance (input) must be a positive number\n')
    covu = input('covu = ')
end

while ndat <= 0 | ndat ~= floor(ndat) | isstr(ndat)
    fprintf('size of data sample must be a positive integer\n')
    ndat = input('ndat = ')
end


while ord <= 0 | ord > 50 | ord ~= floor(ord) | isstr(ord)
    fprintf('order of initial model must be a positive integer and smaller than 50 \n')
    ord = input('ord = ')
end
% end test of arguments ===============================

ychn = 1;                              % output channel
uchn = 2;                              % input  channel

% model for output: dynamic factor (model 1 from database of models)
str = [1 1 2 2; 1 2 0 1];              % factor structure
Eth = [1.81 -0.8187  0.00468 0.00438]; % regression coeficients
Fac = facarxls(ychn, str);             % output factor
Fac.Eth  = Eth;                        % setting factor fields
Fac.cove = covy;                       % noise covariance
Facs{1}  = Fac;                        % array of factors

% model for input: noise
Fac = facarxls(uchn,[]);               % model: noise
Fac.cove = covu;                       % noise covariance (sufficiently high)
Facs{2}  = Fac;

% simulator
Sim = mixconst(Facs, [1 2], 1);        % build simulator


% get data sample
if seed, randn('seed', seed); rand('seed',seed); end
DATA = zeros(2, ndat);                 % pre-allocated DATA

mixsimul(Sim, ndat);

LegendSize = 10;


clf
plot(DATA(1,:)); grid on
xlabel('time'); ylabel('output values')
title('System output');
drawnow


% select richest structure, build richest factor
maxstr = [ones(1,ord), 1+ones(1,ord+1), 0    % richest structure
            1:ord,     0:ord,         1];
Fac0   = facarx(ychn, maxstr);         % richest initial factor


% ==========================================================
% structure estimation with prior knowledge
% ==========================================================
GAIN = [uchn, gain];

pre  = preproc( {'scale' []} );       % data are preprocessed


pri  = {'gain' GAIN}; 


pri  = scalepri(pri, pre, ychn);      % preprocess prior knowledge


nbest = 5;                            % number of "best" regressors
nruns = 100;                          % number of estimation runs

[Fac, vll] = pristr(Fac0, pri, [], nbest, nruns);
disp('WITH PRIOR KNOWLEDGE');
pristrd(maxstr, vll);
disp(' ');


Mix0 = mixconst(Fac0, 1, 1);
Mix  = mixestqb(Mix0, 1, ndat, 50);
F    = Mix.Facs{1};

[optstr, lhs] = facstr(F, Fac0, [], nbest, nruns);    %, uchn, 0.9, ord);
disp('WITHOUT PRIOR KNOWLEDGE');
pristrd(maxstr, lhs);


legend('structure estimation summary',...
'simulated model structure',...
num2str(str), ...
'richest model structure',...
num2str(maxstr),...
'estimated model structure with prior knowledge',...
num2str(Fac.str),...
'estimated model structure without prior knowledge',...
num2str(optstr), 0 );
figure(1);




%plt('zgain_');
dial2
end % ====================================
