% zpriest.m
% FACTOR STRUCTURE ESTIMATION WITH PRIOR KNOWLEDGE OF GAIN
%
% Design : P. Nedoma
% Updated: November 2004
% Projects : GACR 102/03/0049, AVCR S1075351, 1ET100750401,
%            MINERVA 110330-CP-1-2003-1-ES-MINERVA-M


prodini
echo off
idef = [];

while 1 % ==============================================

if isempty(idef)
ndat  = 40;                          % sample size       
covu  = 0.1;
covy  = 0.1*0.1;  
seed  = 0;      
gain  = [0.9 1.1];
ord   = 6;
diaCth   = 1;
end

clc
fprintf('FACTOR STRUCTURE ESTIMATION WITH PRIOR KNOWLEDGE OF GAIN\n');
fprintf('\n--- System: SISO dynamic, 2nd order, open loop --------\n');
fprintf('output noise covariance ->   covy   = %g;\n',      covy);
fprintf('output noise covariance ->   covu   = %g;\n',      covu);

fprintf('\n---  Measurements ------------------------------\n');
fprintf('size                    ->   ndat  = %i;\n',       ndat);
fprintf('seed                    ->   seed  = %i;\n',       seed);

fprintf('\n---  Prior knowledge ---------------------------\n');
fprintf('gain range              ->   gain  = [%g %g];\n',   gain);

fprintf('\n--- initial model to be estimated ---------------\n');
fprintf('diagonal of par. covar. -> diaCth  = %g;\n',   diaCth);

fprintf('--------------------------------------------------\n');

dial1;

ychn = 1;                              % output channel
uchn = 2;                              % input  channel

% model for output: dynamic factor (model 1 from database of models)
str = [1 1 2 2; 1 2 0 1];              % factor structure
Eth = [1.81 -0.8187  0.00468 0.00438]; % regression coeficients
Fac = facarxls(ychn, str);             % output factor
Fac.Eth  = Eth;                        % setting factor fields
Fac.cove = covy;                       % noise covariance
Facs{1}  = Fac;                        % array of factors

% model for input: noise
Fac      = facarxls(uchn,[]);          % model: noise
Fac.cove = covu;                       % noise covariance (sufficiently high)
Facs{2}  = Fac;

% simulator
Sim = mixconst(Facs, [1 2], 1);        % build simulator


% get data sample
if seed, randn('seed', seed); rand('seed',seed); end
DATA = zeros(2, ndat);                 % pre-allocated DATA

mixsimul(Sim, ndat);

sub = 120; 
LegendSize = 11;

% ==========================================================
% structure estimation with prior knowledge
% ==========================================================
GAIN = [uchn, gain];

pre  = preproc( {'scale' []} );        % data are preprocessed


pri  = {'gain' GAIN}; 


pri  = scalepri(pri, pre, ychn);       % preprocess prior knowledge


Fac = facarxls(ychn, str);
nphi = length(Fac.Cth);
Fac.Cth = diag(nphi + zeros(1, nphi));
Fac = arx2arx(Fac);

Mix0 = mixconst(Fac, 1, 1);


Fac  = prior(Fac, pri);
Mix = mixconst(Fac, 1, 1);



E=[]; E0=[];
for TIME = 4:ndat
    Mix0 = mixestim(Mix0,1);
    Mix  = mixestim(Mix,1);
    F0   = arx2arx(Mix0.Facs{1});
    F    = arx2arx(Mix.Facs{1});
    E0   = [E0, F0.Eth'];
    E    = [E , F.Eth'];
end

clf
for i=[3,4]
   sub=sub+1; subplot(sub);
   plot(E0(i, ':'),':'); hold on; plot(E(i,:),'r');
   plot([1 ndat], [Eth(i) Eth(i)],':m');
if i==3, H=legend('b0 trajectory', 'with prior');
else     H=legend('b1 trajectory', 'with prior');
end
end
figure(1);

%plt('zpriest_');

dial2
end % ====================================
