% priestn.m
% FACTOR STRUCTURE ESTIMATION WITH DIFFERENT PRIOR KNOWLEDGE
%
% Designed : P. Nedoma
% Updated  : November 2004
% Projects : GACR 102/03/0049, AVCR S1075351, 1ET100750401,
%            MINERVA 110330-CP-1-2003-1-ES-MINERVA-M

prodini
echo off
idef = [];

while 1 % ==============================================

if isempty(idef)
ndat  = 40;                          % sample size       
covu  = 0.01;
covy  = 0.01;  
seed  = 0;      
ord   = 6;
diaCth = 10;
typ    = 1;
end 

clc
fprintf('\nFACTOR STRUCTURE ESTIMATION WITH DIFFERENT PRIOR KNOWLEDGE\n');
fprintf('\n--- System: dynamic, 2nd order, open loop --------\n');
fprintf('output noise covariance ->   covy   = %g;\n',      covy);
fprintf('output noise covariance ->   covu   = %g;\n',      covu);

fprintf('\n---  Measurements ------------------------------\n');
fprintf('size                    ->   ndat  = %i;\n',       ndat);
fprintf('seed                    ->   seed  = %i;\n',       seed);

fprintf('\n--- initial model to be estimated ---------------\n');
fprintf('diagonal of par. covar. -> diaCth  = %g;\n',   diaCth);

fprintf('\n---  Prior knowledge ---------------------------\n');
fprintf('prior knowledge         ->   typ   = %i;\n',    typ);

fprintf('%s\n%s\n%s\n%s\n',...
'1 : gain',...
'2 : cut off frequency',...
'3 : time constants',...
'4 : frequency characteristics',...
'5 : dynamic data',...
'6 : data envelope');
fprintf('--------------------------------------------------\n');

dial1;

ychn = 1;                              % output channel
uchn = 2;                              % input  channel

Eth =[2.7145   -2.4562    0.7408    0.0002    0.0006    0.0001];
str = [...
     1     1     1     2     2     2
     1     2     3     0     1     2 ];
Fac = facarxls(ychn, str);             % output factor
Fac.Eth  = Eth;                        % setting factor fields
Fac.cove = covy;                       % noise covariance
Facs{1}  = Fac;                        % array of factors
Fac1     = Fac;

% model for input: noise
Fac      = facarxls(uchn,[]);          % model: noise
Fac.cove = covu;                       % noise covariance (sufficiently high)
Facs{2}  = Fac;

% simulator
Sim = mixconst(Facs, [1 2], 1);        % build simulator


% get data sample
if seed, randn('seed', seed); rand('seed',seed); end
DATA = zeros(2, ndat);                 % pre-allocated DATA

mixsimul(Sim, ndat);

sub = 130; 
LegendSize = 11;

% ==========================================================
% structure estimation with prior knowledge
% ==========================================================

st  = 0.1;   
switch typ
   case 1, 
      gain =  0.9576;
      ssig =  sqrt(covy);
      pri  = {'gain', [uchn gain-2*ssig gain+2*ssig] };
   case 2
      wcutc = 5.3269;                        % frequency in radians
      cut   = wcutc/2/pi;                    % continuous, in Hertz
      pri   = { 'st' 0.1 'cut' [uchn cut] };
   case 3
      tc   = [0.7249 0.7396 1.2113 1.2358];
      pri  = { 'st' st 'tc', [uchn tc] };
   case 4
      freq  = 0.1370;                        % frequency
      ampl  = [0.4641 0.4659];               % amplitude range
      phase = 10.8493;                       % phase
      pri   = { 'st' st 'ampl' [uchn freq ampl phase] };
   case 5
      nddat = 50;
      S = mixconst(Fac1, 1, 1);
      D = DATA;
      DATA = [zeros(1, nddat); sin(1:nddat)];
      mixsimul(S, nddat);
      pri  = {'d' DATA};
      sigscan;
      DATA = D;
   case 6
      nrep  = 20;
      nddat = 50;
      DATA  = [zeros(1, nddat); ...
               zeros(1,10), 1 + zeros(1, nddat-10)];
      for i=1:nrep
          mixsimul(Sim, nddat);               % get data sample
          if i==1, data = DATA(1,:);
          else     data = [data; DATA(1,:)];
          end
      end
     % form data envelope
      m  = mean(data);
      s  = std(data);
      y1 = m + 2*s;
      y2 = m - 2*s;


u  = DATA(2,:);
dhigh = [y1;u];
dlow  = [y2;u];

pri   = {'fdata', {dlow dhigh} };




sb = 220; 

clf; hold off

sb=sb+1; subplot(sb);
plot(data'); 
H=legend('learning trajectories');
drawnow

sb=sb+1; subplot(sb);
plot(y1,'b'); hold on; plot(y2,'r');
plot(m,':');  grid on;
H=legend('learning data envelope');
drawnow


sb=sb+1; subplot(sb);
plot(DATA(2,:)'); grid on
H=legend('last input');
drawnow


sb=sb+1; subplot(sb);
plot(DATA(1,:)'); grid on
H=legend('last output');
drawnow

figure(1);
disp('Press a key ...');
pause

end % ----------------------------------------------



pre  = preproc( {'scale' []} );        % data are preprocessed
pri  = scalepri(pri, pre, ychn);       % preprocess prior knowledge

len       = length(str);
Fac0      = facarxls(ychn, str);
Fac0.dfm  = 100;
Fac0.cove = diag(covy + zeros(1,len));
Fac0.Cth  = diag(diaCth  + zeros(1,len));
Fac0      = arx2arx(Fac0);
Mix0      = mixconst(Fac0, 1, 1);


Fac  = prior(Fac0, pri);
Mix0 = mixconst(Fac0, 1, 1);
Mix  = mixconst(Fac,  1, 1);



E=[]; E0=[];
for TIME = 4:ndat
    Mix0 = mixestim(Mix0,1);
    Mix  = mixestim(Mix,1);
    F0   = arx2arx(Mix0.Facs{1});
    F    = arx2arx(Mix.Facs{1});
    E0   = [E0, F0.Eth'];
    E    = [E , F.Eth'];
end

clf
for i=[4:6]
   sub=sub+1; subplot(sub);
   plot(E0(i, ':'),':'); hold on; plot(E(i,:),'r');
   plot([1 ndat], [Eth(i) Eth(i)],':m');
if     i==3, H=legend('b0 trajectory', 'with prior');
elseif i==4  H=legend('b1 trajectory', 'with prior');
else         H=legend('b2 trajectory', 'with prior');
end
end
figure(1);
%plt('zpriestn_');

dial2
end % ====================================
